/*******************************************************************************
 * Copyright (c) 2012-2014 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - Initial API and implementation
 * Carsten Karbach, FZ Juelich
 *******************************************************************************/
package org.eclipse.ptp.internal.rm.lml.monitor.ui;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;

public class MonitorImages {
	private static final String NAME_PREFIX = LMLMonitorUIPlugin.PLUGIN_ID + '.';
	private static final int NAME_PREFIX_LENGTH = NAME_PREFIX.length();

	// The plugin registry
	private static ImageRegistry imageRegistry = new ImageRegistry();

	// Subdirectory (under the package containing this class) where 16 color images are
	private static URL fgIconBaseURL;
	static {
		fgIconBaseURL = Platform.getBundle(LMLMonitorUIPlugin.getUniqueIdentifier()).getEntry("/icons/"); //$NON-NLS-1$
	}

	private static final String T_TABS = "eview16/"; //$NON-NLS-1$

	public static String IMG_STARTED = NAME_PREFIX + "monitor_started.gif"; //$NON-NLS-1$
	public static String IMG_FORCEUPDATE = NAME_PREFIX + "monitor_force_update.png"; //$NON-NLS-1$

	public static final ImageDescriptor DESC_STARTED = createManaged(T_TABS, IMG_STARTED);
	public static final ImageDescriptor FORCE_STARTED = createManaged(T_TABS, IMG_FORCEUPDATE);

	private static ImageDescriptor createManaged(String prefix, String name) {
		return createManaged(imageRegistry, prefix, name);
	}

	private static ImageDescriptor createManaged(ImageRegistry registry, String prefix, String name) {
		ImageDescriptor result = ImageDescriptor.createFromURL(makeIconFileURL(prefix, name.substring(NAME_PREFIX_LENGTH)));
		registry.put(name, result);
		return result;
	}

	public static Image get(String key) {
		return imageRegistry.get(key);
	}

	private static URL makeIconFileURL(String prefix, String name) {
		StringBuffer buffer = new StringBuffer(prefix);
		buffer.append(name);
		try {
			return new URL(fgIconBaseURL, buffer.toString());
		} catch (MalformedURLException e) {
			LMLMonitorUIPlugin.log(e.getLocalizedMessage());
			return null;
		}
	}

	/**
	 * Helper method to access the image registry from the JavaPlugin class.
	 */
	static ImageRegistry getImageRegistry() {
		return imageRegistry;
	}
}
